<?php

/**
 * @file
 * Contains \Drupal\saml_sp\Form\SamlSpIdpAdd.
 */


namespace Drupal\saml_sp\Form;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;

class IdpForm extends EntityForm {

  /**
   * @param \Drupal\Core\Entity\Query\QueryFactory $entity_query
   *   The entity query.
   */
  public function __construct(QueryFactory $entity_query) {
    $this->entityQuery = $entity_query;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.query')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $idp = $this->entity;

    $form['idp_metadata'] = array(
      '#type' => 'textarea',
      '#title'  => t('XML Metadata'),
      '#description' => t('Paste in the metadata provided by the Identity Provider here and the form will be automatically filled out, or you can manually enter the information.'),
    );
    $form['#attached']['library'][] = 'saml_sp/idp_form';

    $form['idp'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
    );

    $form['idp']['label'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => $idp->label(),
      '#description' => t('The human-readable name of this IDP. This text will be displayed to administrators who can configure SAML.'),
      '#required' => TRUE,
      '#size' => 30,
      '#maxlength' => 30,
    );

    $form['idp']['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $idp->id(),
      '#maxlength' => 32,
      '#machine_name' => array(
        'exists' => 'saml_sp_idp_load',
        'source' => array('idp', 'label'),
      ),
      '#description' => t('A unique machine-readable name for this IDP. It must only contain lowercase letters, numbers, and underscores.'),
    );

    $form['idp']['entity_id'] = array(
      '#type' => 'textfield',
      '#title' => t('Entity ID'),
      '#description' => t('The entityID identifier which the Identity Provider will use to identiy itself by, this may sometimes be a URL.'),
      '#default_value' => $idp->entity_id(),
      '#maxlength' => 255,
    );

    $form['idp']['app_name'] = array(
      '#type' => 'textfield',
      '#title' => t('App name'),
      '#description' => t('The app name is provided to the Identiy Provider, to identify the origin of the request.'),
      '#default_value' => $idp->app_name(),
      '#maxlength' => 255,
    );

    $fields = array('mail' => t('Email'));
    if (!empty($extra_fields)) {
      foreach ($extra_fields as $value) {
        $fields[$value] = $value;
      }
    }

    $form['idp']['nameid_field'] = array(
      '#type' => 'select',
      '#title' => t('NameID field'),
      '#description' => t('Mail is usually used between IdP and SP, but if you want to let users change the email address in IdP, you need to use a custom field to store the ID.'),
      '#options' => $fields,
      '#default_value' => $idp->nameid_field(),
    );

    // The SAML Login URL and x.509 certificate must match the details provided
    // by the IDP.
    $form['idp']['login_url'] = array(
      '#type' => 'textfield',
      '#title' => t('IDP Login URL'),
      '#description' => t('Login URL of the Identity Provider server.'),
      '#default_value' => $idp->login_url(),
      '#required' => TRUE,
      '#max_length' => 255,
    );

    $form['idp']['logout_url'] = array(
      '#type' => 'textfield',
      '#title' => t('IDP Logout URL'),
      '#description' => t('Logout URL of the Identity Provider server.'),
      '#default_value' => $idp->logout_url(),
      '#required' => TRUE,
      '#max_length' => 255,
    );

    $form['idp']['x509_cert'] = array(
      '#type' => 'textarea',
      '#title' => t('x.509 Certificate'),
      '#description' => t('Enter the application certificate provided by the IdP.'),
      '#default_value' => $idp->x509_cert(),
      '#required' => TRUE,
      '#max_length' => 1024,
    );

    $refs = saml_sp_authn_context_class_refs();
    $authn_context_class_ref_options = array(
      $refs['urn:oasis:names:tc:SAML:2.0:ac:classes:Password']                   => t('User Name and Password'),
      $refs['urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport'] => t('Password Protected Transport'),
      $refs['urn:oasis:names:tc:SAML:2.0:ac:classes:TLSClient']                  => t('Transport Layer Security (TLS) Client'),
      $refs['urn:oasis:names:tc:SAML:2.0:ac:classes:X509']                       => t('X.509 Certificate'),
      $refs['urn:federation:authentication:windows']                             => t('Integrated Windows Authentication'),
      $refs['urn:oasis:names:tc:SAML:2.0:ac:classes:Kerberos']                   => t('Kerberos'),
    );

    $form['idp']['authn_context_class_ref'] = array(
      '#type'           => 'checkboxes',
      '#title'          => t('Authentication Methods'),
      '#description'    => t('What authentication methods would you like to use with this IdP?'),
      '#default_value'  => $idp->authn_context_class_ref(),
      '#options'        => $authn_context_class_ref_options,
      '#required' => TRUE,
    );
    if (empty($idp->id)) {
      // we haven't saved the IdP yet so don't show metadata
      return $form;
    }
    /*
    if ( module_exists('devel')) {
      $form['saml_sp__debug'] = array(
        '#type'             => 'checkbox',
        '#title'            => t('Debug'),
        '#description'      => t('Works with Devel module to display SAML requests and Responses for review.'),
        //'#default_value'    => variable_get('saml_sp__debug', FALSE),
      );
    }/**/
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $idp = $this->entity;
    $values = $form_state->getValues();
    foreach ($values['idp'] AS $key => $value) {
      $method = 'set' . ucfirst($key);
      $idp->$method($value);
    }

    $status = $idp->save();

    if ($status) {
      drupal_set_message($this->t('Saved the %label Example.', array(
        '%label' => $idp->label(),
      )));
    }
    else {
      drupal_set_message($this->t('The %label Identity Provider was not saved.', array(
        '%label' => $example->label(),
      )));
    }

    $form_state->setRedirect('entity.idp.collection');
  }

  public function exist($id) {
    $entity = $this->entityQuery->get('idp')
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }
}
